CREATE OR REPLACE FUNCTION SOMA_IMPOSTO(
   P_CDEMPRESA    IN MAPAAPURAC.CDEMPRESA%TYPE,
   P_CDIMPOSTO    IN MAPAAPURAC.CDIMPOSTO%TYPE,
   P_NRSEQUMAPAPU IN MAPAAPURAC.NRSEQUMAPAPU%TYPE,
   P_INSCRICAO    IN MAPAAPURAC.CDINSCESTA%TYPE,
   P_DTFINAL      IN MAPAAPURAC.DTFIMPERIODO%TYPE,
   P_DTINICIAL    IN MAPAAPURAC.DTINIPERIODO%TYPE,
   P_NOTASMAPA    IN MAPAAPURAC.VRTOTAIMPOS%TYPE,
   P_CHAMADAPROC  IN VARCHAR2) RETURN VARCHAR
AS
   V_VRTOTADEBI         NUMBER;
   V_VRTOTACRED         NUMBER;
   V_VRSALDODEBI        NUMBER;
   V_VRSALDOCRED        NUMBER;
   V_IDAPURACAO         VARCHAR2(01);
   V_NFFORAPERIODO      VARCHAR2(01);
   V_UTFORMULA          VARCHAR2(01);
   V_BOIMPOSAPUR        BOOLEAN;
   V_BOATUCTCTBL        BOOLEAN;
   V_ANO                VARCHAR2(04);
   V_MES                VARCHAR2(02);
   V_DIA                VARCHAR2(02);
   V_NRSEQITMAPA        NUMBER;
   V_IDORGARRECADNFE    VARCHAR2(01);
   V_IDFORMAPURNFE      VARCHAR2(01);
   V_IDIMPORECUPNFE     VARCHAR2(01);
   V_DTININFFORPER      DATE;
   V_IDNFFORAPERAPU     VARCHAR2(01);
   V_RETORNO            VARCHAR2(03);
   V_VROUTRDEBI         NUMBER;
   V_VRESTOCRED         NUMBER;
   V_VROUTRCRED         NUMBER;
   V_VRESTODEBI         NUMBER;
   V_VRDEDUCAO          NUMBER;
   V_VRDIFALIQU         NUMBER;
   V_VRCONTSAID         NUMBER;
   V_VRBASESAID         NUMBER;
   V_VRCONTENTR         NUMBER;
   V_VRBASEENTR         NUMBER;
   V_VRIMPOSAID         NUMBER;
   V_VRIMPOENTR         NUMBER;
   V_IDCREDDEBNORM      VARCHAR2(01);
   V_IDOUTCREDDEB       VARCHAR2(01);
   V_SEQMAPAANT         VARCHAR2(10);
   V_VRSALDOCREDANT     NUMBER;
   V_IDNFFORAPERAPUIMP  VARCHAR2(01);
   V_IDSOMAFECP         VARCHAR2(01);
   V_VRSALDOCREDANTSUBA NUMBER;
   V_EMPRRELAC          VARCHAR2(01);
   V_IDENVBCSPEDGIA     VARCHAR2(01);
BEGIN

   V_BOIMPOSAPUR := FALSE;
   V_BOATUCTCTBL := FALSE;

   BEGIN
      SELECT IDSOMAFECP
        INTO V_IDSOMAFECP
        FROM AJUSTEFECP
       WHERE CDEMPRESA    = P_CDEMPRESA
         AND CDINSCESTA   = P_INSCRICAO
         AND CDIMPOSTO    = P_CDIMPOSTO;
   EXCEPTION
      WHEN NO_DATA_FOUND THEN
         V_IDSOMAFECP := 'S';
   END;
   V_IDSOMAFECP := NVL(V_IDSOMAFECP,'S');

   BEGIN
      SELECT MAX(NVL(IDENVBCSPEDGIA,'N'))
        INTO V_IDENVBCSPEDGIA
        FROM INSCRESTAD
       WHERE CDEMPRESA  = P_CDEMPRESA
         AND CDINSCESTA = P_INSCRICAO;
   EXCEPTION
      WHEN NO_DATA_FOUND THEN
         V_IDENVBCSPEDGIA := 'N';
   END;

   /******** Busca o orgo arrecadador, a forma de apurao do imposto
             e se o mesmo  recupervel ********/
   BEGIN
      SELECT IDORGARRECADNFE, IDFORMAPURNFE, IDIMPORECUPNFE
        INTO V_IDORGARRECADNFE, V_IDFORMAPURNFE, V_IDIMPORECUPNFE
        FROM IMPOSTONFE I
       WHERE CDIMPOSTONFE = P_CDIMPOSTO;
   EXCEPTION
      WHEN NO_DATA_FOUND THEN
         V_IDORGARRECADNFE := NULL;
         V_IDFORMAPURNFE   := NULL;
         V_IDIMPORECUPNFE  := NULL;
   END;

   FOR C_IMPOSRELAC IN (SELECT CDIMPOSAPUR, IDATUCCTBITIMP
                          FROM IMPOSTO
                         WHERE CDIMPOSTONFE = P_CDIMPOSTO) LOOP

      IF C_IMPOSRELAC.CDIMPOSAPUR IS NOT NULL THEN
         V_BOIMPOSAPUR := TRUE;
      END IF;

      IF C_IMPOSRELAC.IDATUCCTBITIMP = 'S' THEN
         V_BOATUCTCTBL := TRUE;
      END IF;
   END LOOP;

   BEGIN
      SELECT DTININFFORPER, IDNFFORAPERAPU
        INTO V_DTININFFORPER, V_IDNFFORAPERAPU
        FROM PARAMFISCAL;
   EXCEPTION
      WHEN NO_DATA_FOUND THEN
         V_DTININFFORPER  := '';
         V_IDNFFORAPERAPU := '';
   END;

   BEGIN
      SELECT MAX(IDNFFORAPERAPU)
        INTO V_IDNFFORAPERAPUIMP
        FROM IMPOSTO
       WHERE CDIMPOSTONFE = P_CDIMPOSTO;
   EXCEPTION
      WHEN NO_DATA_FOUND THEN
         V_IDNFFORAPERAPUIMP := '';
   END;

   IF V_IDNFFORAPERAPUIMP <> 'I' THEN
      V_IDNFFORAPERAPU := V_IDNFFORAPERAPUIMP;
   END IF;

   CASE V_IDFORMAPURNFE
      WHEN 'P' THEN
         /******** Apurao pela Data de Pagamento ********/
         V_RETORNO := APURA_DATA_PAGAMENTO(P_NRSEQUMAPAPU, P_CDEMPRESA, P_CDIMPOSTO, P_INSCRICAO, P_DTFINAL, P_DTINICIAL, V_BOIMPOSAPUR, V_IDORGARRECADNFE, V_DTININFFORPER, V_IDNFFORAPERAPU);
         V_IDAPURACAO    := SUBSTR(V_RETORNO,1,1);
         V_NFFORAPERIODO := SUBSTR(V_RETORNO,2,1);
      WHEN 'C' THEN
         /******** Apurao pela Data de Competncia ********/
         V_RETORNO := APURA_DATA_COMPETENCIA(P_NRSEQUMAPAPU, P_CDEMPRESA, P_CDIMPOSTO, P_INSCRICAO, P_DTFINAL, P_DTINICIAL, V_BOIMPOSAPUR, V_IDORGARRECADNFE, V_DTININFFORPER, V_IDNFFORAPERAPU);
         V_IDAPURACAO    := SUBSTR(V_RETORNO,1,1);
         V_NFFORAPERIODO := SUBSTR(V_RETORNO,2,1);
         V_IDOUTCREDDEB  := SUBSTR(V_RETORNO,3,1);
      WHEN 'D' THEN
         /******** Apurao pela Data de Entrada/Saida ********/
         V_RETORNO := APURA_DATA_ENTRSAID(P_NRSEQUMAPAPU, P_CDEMPRESA, P_CDIMPOSTO, P_INSCRICAO, P_DTFINAL, P_DTINICIAL, V_BOIMPOSAPUR, V_IDORGARRECADNFE, V_DTININFFORPER, V_IDNFFORAPERAPU);
         V_IDAPURACAO    := SUBSTR(V_RETORNO,1,1);
         V_NFFORAPERIODO := SUBSTR(V_RETORNO,2,1);
      WHEN 'E' THEN
         V_RETORNO := APURA_DATA_EMISSAO(P_NRSEQUMAPAPU, P_CDEMPRESA, P_CDIMPOSTO, P_INSCRICAO, P_DTFINAL, P_DTINICIAL, V_BOIMPOSAPUR, V_IDORGARRECADNFE, V_DTININFFORPER, V_IDNFFORAPERAPU);
         V_IDAPURACAO    := SUBSTR(V_RETORNO,1,1);
         V_NFFORAPERIODO := SUBSTR(V_RETORNO,2,1);
   END CASE;

   IF V_IDNFFORAPERAPU = 'S' THEN
      UPDATE IMPOSTONF
         SET DTREFMAPAPU   = P_DTINICIAL
       WHERE (NRSEQUMAPAPU = P_NRSEQUMAPAPU OR
              NRSEQMAPOIMP = P_NRSEQUMAPAPU);

      UPDATE IMPBAIXAREC
         SET DTREFMAPAPU   = P_DTINICIAL
       WHERE (NRSEQUMAPAPU = P_NRSEQUMAPAPU OR
              NRSEQMAPOIMP = P_NRSEQUMAPAPU);
   ELSE
      UPDATE IMPOSTONF
         SET DTREFMAPAPU   = ''
       WHERE (NRSEQUMAPAPU = P_NRSEQUMAPAPU OR
              NRSEQMAPOIMP = P_NRSEQUMAPAPU);

      UPDATE IMPBAIXAREC
         SET DTREFMAPAPU   = ''
       WHERE (NRSEQUMAPAPU = P_NRSEQUMAPAPU OR
              NRSEQMAPOIMP = P_NRSEQUMAPAPU);
   END IF;

   BEGIN
      SELECT NVL(SUM(DECODE(IDTIPOITMAPR,'OD',VRITEMMAPR,0)),0) AS VROUTRDEBI,
             NVL(SUM(DECODE(IDTIPOITMAPR,'EC',VRITEMMAPR,0)),0) AS VRESTOCRED,
             NVL(SUM(DECODE(IDTIPOITMAPR,'OC',VRITEMMAPR,0)),0) AS VROUTRCRED,
             NVL(SUM(DECODE(IDTIPOITMAPR,'ED',VRITEMMAPR,0)),0) AS VRESTODEBI,
             NVL(SUM(DECODE(IDTIPOITMAPR,'DD',VRITEMMAPR,0)),0) AS VRDEDUCAO,
             NVL(SUM(DECODE(IDTIPOITMAPR,'DA',VRITEMMAPR,0)),0) AS VRDIFALIQU
        INTO V_VROUTRDEBI, V_VRESTOCRED, V_VROUTRCRED, V_VRESTODEBI,
             V_VRDEDUCAO, V_VRDIFALIQU
        FROM ITMAPARECUP
       WHERE NRSEQUMAPAPU = P_NRSEQUMAPAPU;
   EXCEPTION
      WHEN NO_DATA_FOUND THEN
          V_VROUTRDEBI := 0;
          V_VRESTOCRED := 0;
          V_VROUTRCRED := 0;
          V_VRESTODEBI := 0;
          V_VRDEDUCAO  := 0;
          V_VRDIFALIQU := 0;
   END;

   V_VRCONTENTR := 0;
   V_VRBASEENTR := 0;
   V_VRIMPOENTR := 0;
   V_VRCONTSAID := 0;
   V_VRBASESAID := 0;
   V_VRIMPOSAID := 0;

   /******** Impostos de Apurao Conjunta ********/
   IF V_IDAPURACAO IN ('A','B','T','D') THEN
      BEGIN
         SELECT NVL(SUM(DECODE(IDENTRSAIDOP,'E',VRCONTABILNF,0)),0) AS VRCONTENTR,
                NVL(SUM(DECODE(IDENTRSAIDOP,'E',VRBASECALCNF,0)),0) AS VRBASEENTR,
                NVL(SUM(DECODE(IDENTRSAIDOP,'E',VRIMPOSTONF, 0)),0) AS VRIMPOENTR,
                NVL(SUM(DECODE(IDENTRSAIDOP,'S',VRCONTABILNF,0)),0) AS VRCONTSAID,
                NVL(SUM(DECODE(IDENTRSAIDOP,'S',VRBASECALCNF,0)),0) AS VRBASESAID,
                NVL(SUM(DECODE(IDENTRSAIDOP,'S',VRIMPOSTONF, 0)),0) AS VRIMPOSAID
           INTO V_VRCONTENTR, V_VRBASEENTR, V_VRIMPOENTR,
                V_VRCONTSAID, V_VRBASESAID, V_VRIMPOSAID
           FROM (SELECT NF.IDENTRSAIDOP, I.IDIMPORECUP, ROUND(IP.VRCONTABILNF,2) AS VRCONTABILNF,
                        ROUND(DECODE(IP.CDFISCALNF,'1',IP.VRBASECALCNF,0),2) AS VRBASECALCNF,
                        ROUND(DECODE(IP.CDFISCALNF,'1',IP.VRIMPOSTONF + DECODE(NF.IDENTRSAIDOP,'E',NVL(IP.VRADICFECP,0),DECODE(V_IDSOMAFECP,'N',0,NVL(IP.VRADICFECP,0))),0),2) AS VRIMPOSTONF
                   FROM IMPOSTONF IP, IMPOSTO I, NOTAFISCAL NF
                  WHERE I.CDIMPOSTONFE  = P_CDIMPOSTO
                    AND IP.CDIMPOSTO    = I.CDIMPOSTO
                    AND NF.CDFILIAL     = IP.CDFILIAL
                    AND NF.NRLANCTONF   = IP.NRLANCTONF
                    AND (IP.IDIMPOSTONF <> '3' OR IP.IDIMPOSTONF IS NULL)
                    AND IP.NRSEQMAPOIMP = P_NRSEQUMAPAPU
                  UNION ALL
                 SELECT N.IDENTRSAIDOP, IM.IDIMPORECUP, ROUND(I.VRPROPREC,2) AS VRCONTABIL,
                        ROUND(DECODE(I.CDFISCALNF,'1',I.VRPROPRECBC,0),2) AS VRBASECALC,
                        ROUND(DECODE(I.CDFISCALNF,'1',I.VRPROPRECBC*I.VRALIQNF/100, 0),2) AS VRIMPOSTO
                   FROM IMPBAIXAREC I, NOTAFISCAL N, IMPOSTO IM
                  WHERE IM.CDIMPOSTONFE = P_CDIMPOSTO
                    AND IM.CDIMPOSTO    = I.CDIMPOSTO
                    AND N.CDFILIAL      = I.CDFILIAL
                    AND N.NRLANCTONF    = I.NRLANCTONF
                    AND I.NRSEQMAPOIMP  = P_NRSEQUMAPAPU);
      EXCEPTION
         WHEN NO_DATA_FOUND THEN
            V_VRCONTENTR := 0;
            V_VRBASEENTR := 0;
            V_VRIMPOENTR := 0;
            V_VRCONTSAID := 0;
            V_VRBASESAID := 0;
            V_VRIMPOSAID := 0;
      END;

      UPDATE MAPARECUP
         SET VRIMPCARGATRIBM = 0
       WHERE NRSEQUMAPAPU    = P_NRSEQUMAPAPU;

   ELSIF  V_IDAPURACAO IN ('V','G','E','N') THEN
      BEGIN
         SELECT NVL(SUM(DECODE(IDENTRSAIDOP,'E',VRCONTABILNF,0)),0) AS VRCONTENTR,
                NVL(SUM(DECODE(IDENTRSAIDOP,'E',DECODE(P_CDIMPOSTO,'80',DECODE(IDREGIMEICMSST,'1',VRBASECALCNF,0),VRBASECALCNF),0)),0) AS VRBASEENTR,
                NVL(SUM(DECODE(IDENTRSAIDOP,'E',DECODE(P_CDIMPOSTO,'80',DECODE(IDREGIMEICMSST,'1',VRIMPOSTONF, 0),VRIMPOSTONF), 0)),0) AS VRIMPOENTR,
                NVL(SUM(DECODE(IDENTRSAIDOP,'S',VRCONTABILNF,0)),0) AS VRCONTSAID,
                NVL(SUM(DECODE(IDENTRSAIDOP,'S',VRBASECALCNF,0)),0) AS VRBASESAID,
                NVL(SUM(DECODE(IDENTRSAIDOP,'S',VRIMPOSTONF, 0)),0) AS VRIMPOSAID
           INTO V_VRCONTENTR, V_VRBASEENTR, V_VRIMPOENTR,
                V_VRCONTSAID, V_VRBASESAID, V_VRIMPOSAID
           FROM (SELECT NF.IDENTRSAIDOP, I.IDIMPORECUP, ROUND(IP.VRCONTABILNF,2) AS VRCONTABILNF,
                        ROUND(DECODE(IP.CDFISCALNF,'1',IP.VRBASECALCNF,0),2) AS VRBASECALCNF,
                        ROUND(DECODE(IP.CDFISCALNF,'1',IP.VRIMPOSTONF + DECODE(NF.IDENTRSAIDOP,'E',NVL(IP.VRADICFECP,0),DECODE(V_IDSOMAFECP,'N',0,NVL(IP.VRADICFECP,0))),0),2) AS VRIMPOSTONF,
                        E.IDREGIMEICMSST
                   FROM IMPOSTONF IP, IMPOSTO I, NOTAFISCAL NF, EMPRESA E
                  WHERE I.CDIMPOSTONFE   = P_CDIMPOSTO
                    AND IP.CDIMPOSTO     = I.CDIMPOSTO
                    AND IP.CDFILIAL      = NF.CDFILIAL
                    AND IP.NRLANCTONF    = NF.NRLANCTONF
                    AND ((IP.IDIMPOSTONF <> '3') OR (IP.IDIMPOSTONF IS NULL))
                    AND IP.NRSEQUMAPAPU  = P_NRSEQUMAPAPU
                    AND NF.CDEMPRESA     = E.CDEMPRESA
                  UNION ALL
                 SELECT N.IDENTRSAIDOP, IM.IDIMPORECUP, ROUND(I.VRPROPREC,2) AS VRCONTABIL,
                        ROUND(DECODE(I.CDFISCALNF,'1',I.VRPROPRECBC,0),2) AS VRBASECALC,
                        ROUND(DECODE(I.CDFISCALNF,'1',I.VRPROPRECBC*I.VRALIQNF/100, 0),2) AS VRIMPOSTO,
                        E.IDREGIMEICMSST
                   FROM IMPBAIXAREC I, NOTAFISCAL N, IMPOSTO IM, EMPRESA E
                  WHERE IM.CDIMPOSTONFE = P_CDIMPOSTO
                    AND I.CDIMPOSTO     = IM.CDIMPOSTO
                    AND I.CDFILIAL      = N.CDFILIAL
                    AND I.NRLANCTONF    = N.NRLANCTONF
                    AND I.NRSEQUMAPAPU  = P_NRSEQUMAPAPU
                    AND N.CDEMPRESA     = E.CDEMPRESA);
      EXCEPTION
         WHEN NO_DATA_FOUND THEN
            V_VRCONTENTR := 0;
            V_VRBASEENTR := 0;
            V_VRIMPOENTR := 0;
            V_VRCONTSAID := 0;
            V_VRBASESAID := 0;
            V_VRIMPOSAID := 0;
      END;

      UPDATE MAPARECUP
         SET VRIMPCARGATRIBM = 0
       WHERE NRSEQUMAPAPU    = P_NRSEQUMAPAPU;

   /******** Apurao com Carga Tributria ********/
   ELSE /***** V_IDAPURACAO = 'C' *****/
      BEGIN
         SELECT NVL(VRCONTSAID,0), NVL(VRBASESAID,0), NVL(VRIMPOSAID,0), NVL(VRCONTENTR,0), NVL(VRBASEENTR,0), NVL(VRIMPOENTR,0)
           INTO V_VRCONTSAID, V_VRBASESAID, V_VRIMPOSAID, V_VRCONTENTR, V_VRBASEENTR, V_VRIMPOENTR

           /***** Carga Tributria na Entrada *****/
           FROM (SELECT SUM(VRCONTABILNF) AS VRCONTENTR,
                        SUM(VRBASECALCNF) AS VRBASEENTR,
                        SUM(VRIMPOSTONF)  AS VRIMPOENTR
                   FROM (SELECT ROUND(IP.VRCONTABILNF,2) AS VRCONTABILNF,
                                0 AS VRBASECALCNF,
                                0 AS VRIMPOSTONF
                           FROM IMPOSTONF IP, IMPOSTO I, NOTAFISCAL NF
                          WHERE I.CDIMPOSTONFE   = P_CDIMPOSTO
                            AND NF.IDENTRSAIDOP  = 'E'
                            AND IP.CDIMPOSTO     = I.CDIMPOSTO
                            AND NF.CDFILIAL      = IP.CDFILIAL
                            AND NF.NRLANCTONF    = IP.NRLANCTONF
                            AND ((IP.IDIMPOSTONF <> '3') OR (IP.IDIMPOSTONF IS NULL))
                            AND IP.NRSEQUMAPAPU  = P_NRSEQUMAPAPU
                          UNION ALL
                         SELECT 0 AS VRCONTABIL,
                                NVL(VALOR_IMPOSTO_NFE_CARGA(N.CDFILIAL, N.NRLANCTONF, 'B', 'I', '1'),0) +
                                        NVL(DECODE(V_IDOUTCREDDEB,'S',0,DECODE(V_IDENVBCSPEDGIA,'N',0,VALOR_IMPOSTO_NFE_CARGA(N.CDFILIAL,N.NRLANCTONF,'B','C','1'))),0) AS VRBASECALCNF,
                                NVL(VALOR_IMPOSTO_NFE_CARGA(N.CDFILIAL, N.NRLANCTONF, 'I', 'I', '1'),0) +
                                        NVL(VALOR_FECP_NFE(N.CDFILIAL, N.NRLANCTONF, '00', 'I', '1'),0) +
                                        NVL(DECODE(V_IDOUTCREDDEB,'S',0,DECODE(V_IDENVBCSPEDGIA,'N',0,VALOR_IMPOSTO_NFE_CARGA(N.CDFILIAL,N.NRLANCTONF,'I','C','1'))),0) AS VRIMPOSTONF  
                           FROM (SELECT DISTINCT IP.CDFILIAL, IP.NRLANCTONF
                                   FROM IMPOSTONF IP, IMPOSTO I, NOTAFISCAL NF
                                  WHERE I.CDIMPOSTONFE   = P_CDIMPOSTO
                                    AND NF.IDENTRSAIDOP  = 'E'
                                    AND IP.CDIMPOSTO     = I.CDIMPOSTO
                                    AND NF.CDFILIAL      = IP.CDFILIAL
                                    AND NF.NRLANCTONF    = IP.NRLANCTONF
                                    AND ((IP.IDIMPOSTONF <> '3') OR (IP.IDIMPOSTONF IS NULL))
                                    AND IP.NRSEQUMAPAPU  = P_NRSEQUMAPAPU)N))A,

                 /***** Carga Tributria na Sada *****/
                 (SELECT SUM(VRCONTABILNF) AS VRCONTSAID,
                         SUM(VRBASECALCNF) AS VRBASESAID,
                         SUM(VRIMPOSTONF)  AS VRIMPOSAID
                    FROM (SELECT ROUND(IP.VRCONTABILNF,2) AS VRCONTABILNF,
                                 ROUND(DECODE(IP.CDFISCALNF,'1',IP.VRBASECALCNF,0),2) AS VRBASECALCNF,
                                 0  AS VRIMPOSTONF
                            FROM IMPOSTONF IP, IMPOSTO I, NOTAFISCAL NF
                           WHERE I.CDIMPOSTONFE  = P_CDIMPOSTO
                             AND NF.IDENTRSAIDOP = 'S'
                             AND IP.CDIMPOSTO    = I.CDIMPOSTO
                             AND NF.CDFILIAL     = IP.CDFILIAL
                             AND NF.NRLANCTONF   = IP.NRLANCTONF
                             AND ((IP.IDIMPOSTONF <> '3') OR (IP.IDIMPOSTONF IS NULL))
                             AND IP.NRSEQUMAPAPU = P_NRSEQUMAPAPU
                           UNION ALL
                          SELECT 0 AS VRCONTABILNF,   
                                 CASE WHEN N.IDSISTEMA = 'FIS'
                                           THEN VALOR_IMPOSTO_NFE_CARGA(N.CDFILIAL,N.NRLANCTONF,'B','C','1')
                                           ELSE NVL(VALOR_IMPOSTO_NFE_CARGA(N.CDFILIAL, N.NRLANCTONF, 'B', 'I', '1'),0) +
                                                NVL(DECODE(V_IDOUTCREDDEB,'S',0,DECODE(V_IDENVBCSPEDGIA,'N',0,VALOR_IMPOSTO_NFE_CARGA(N.CDFILIAL,N.NRLANCTONF,'B','C','1'))),0)
                                      END AS VRBASECALCNF,
                                 NVL(VALOR_IMPOSTO_NFE_CARGA(N.CDFILIAL, N.NRLANCTONF, 'I', 'I', '1'),0) +
                                             DECODE(V_IDSOMAFECP,'S',NVL(VALOR_FECP_NFE(N.CDFILIAL, N.NRLANCTONF, '00', 'I', '1'),0),0) +
                                             NVL(DECODE(N.IDSISTEMA,'FIS',VALOR_IMPOSTO_NFE_CARGA(N.CDFILIAL,N.NRLANCTONF,'I','C','1'),
                                                              DECODE(V_IDOUTCREDDEB,'S',0,VALOR_IMPOSTO_NFE_CARGA(N.CDFILIAL,N.NRLANCTONF,'I','C','1'))),0) AS VRIMPOSTONF
                            FROM (SELECT DISTINCT IP.CDFILIAL, IP.NRLANCTONF, NF.IDSISTEMA
                                   FROM IMPOSTONF IP, IMPOSTO I, NOTAFISCAL NF
                                  WHERE I.CDIMPOSTONFE   = P_CDIMPOSTO
                                    AND NF.IDENTRSAIDOP  = 'S'
                                    AND IP.CDIMPOSTO     = I.CDIMPOSTO
                                    AND NF.CDFILIAL      = IP.CDFILIAL
                                    AND NF.NRLANCTONF    = IP.NRLANCTONF
                                    AND ((IP.IDIMPOSTONF <> '3') OR (IP.IDIMPOSTONF IS NULL))
                                    AND IP.NRSEQUMAPAPU  = P_NRSEQUMAPAPU)N))B;
      EXCEPTION
         WHEN NO_DATA_FOUND THEN
            V_VRCONTENTR := 0;
            V_VRBASEENTR := 0;
            V_VRIMPOENTR := 0;
            V_VRCONTSAID := 0;
            V_VRBASESAID := 0;
            V_VRIMPOSAID := 0;
      END;

      /******** Atualiza o Valor da Carga Tributria no Mapa ********/
      UPDATE MAPARECUP
         SET VRIMPCARGATRIBM = (SELECT  VRSAIDA - VRENTRADA
                                  FROM (SELECT NVL((SELECT NVL(SUM(ROUND(I.VRIMPCARGATRIBI,2)),0)
                                                      FROM IMPOSTONF I
                                                     WHERE I.NRSEQUMAPAPU = P_NRSEQUMAPAPU
                                                       AND I.CDIMPOSTO    = P_CDIMPOSTO
                                                       AND SUBSTR(I.CDFISCOPERNF,1,1) IN ('5','6','7')),0) AS VRSAIDA,

                                               NVL((SELECT NVL(SUM(ROUND(I.VRIMPCARGATRIBI,2)),0)
                                                      FROM IMPOSTONF I
                                                     WHERE I.NRSEQUMAPAPU = P_NRSEQUMAPAPU
                                                       AND I.CDIMPOSTO    = P_CDIMPOSTO
                                                       AND SUBSTR(I.CDFISCOPERNF,1,1) IN ('1','2','3')),0) AS VRENTRADA
                                          FROM DUAL))
       WHERE NRSEQUMAPAPU = P_NRSEQUMAPAPU;
   END IF;

   /******** Atualiza a conta contbil dos itens do imposto ********/
   IF V_BOATUCTCTBL THEN
      ATUALIZA_CTBL_IMPOSTONF(P_NRSEQUMAPAPU, P_CDIMPOSTO);
   END IF;

   /******** Busca o sequencial do mapa anterior ********/
   V_SEQMAPAANT := NULL;
   BEGIN
      SELECT MAX(M.NRSEQUMAPAPU)
        INTO V_SEQMAPAANT
        FROM MAPAAPURAC M, (SELECT NRSEQUMAPAPU, CDEMPRESA, CDINSCESTA, CDINSCMUNI, CDIMPOSTO
                              FROM MAPAAPURAC
                             WHERE NRSEQUMAPAPU = P_NRSEQUMAPAPU) P
       WHERE M.CDEMPRESA    = P.CDEMPRESA
         AND M.CDIMPOSTO    = P.CDIMPOSTO
         AND (M.CDINSCESTA  = P.CDINSCESTA OR (M.CDINSCESTA IS NULL AND P.CDINSCESTA IS NULL))
         AND (M.CDINSCMUNI  = P.CDINSCMUNI OR (M.CDINSCMUNI IS NULL AND P.CDINSCMUNI IS NULL))
         AND M.NRSEQUMAPAPU < P.NRSEQUMAPAPU;
   EXCEPTION
      WHEN NO_DATA_FOUND THEN
         V_SEQMAPAANT := NULL;
   END;

   IF V_SEQMAPAANT IS NULL THEN
      /******** Para o primeiro mapa mantem o que esta no campo de credito anterior, para considerar alteracoes manuais ********/
      BEGIN
         SELECT VRSLDCRANTE, VRSLDCRANTESUBAPUR
           INTO V_VRSALDOCREDANT, V_VRSALDOCREDANTSUBA
           FROM MAPARECUP
          WHERE NRSEQUMAPAPU = P_NRSEQUMAPAPU;
      EXCEPTION
         WHEN NO_DATA_FOUND THEN
            V_VRSALDOCREDANT     := NULL;
            V_VRSALDOCREDANTSUBA := NULL;
      END;
   ELSE
      /******** Caso no seja o primeiro mapa, busca o saldo credor do mapa anterior ********/
      BEGIN
         SELECT VRSALDOCRED, VRSLDCRANTESUBAPUR
           INTO V_VRSALDOCREDANT, V_VRSALDOCREDANTSUBA
           FROM MAPARECUP
          WHERE NRSEQUMAPAPU = V_SEQMAPAANT;
      EXCEPTION
         WHEN NO_DATA_FOUND THEN
            V_VRSALDOCREDANT     := NULL;
            V_VRSALDOCREDANTSUBA := NULL;
      END;
   END IF;
   V_VRSALDOCREDANT     := NVL(V_VRSALDOCREDANT,0);
   V_VRSALDOCREDANTSUBA := NVL(V_VRSALDOCREDANTSUBA,0);

   IF (V_IDIMPORECUPNFE = 'N') AND (P_CDIMPOSTO <> '80') THEN
      V_VRIMPOSAID := NVL(V_VRIMPOSAID,0)+NVL(V_VRIMPOENTR,0);
      V_VRIMPOENTR := 0;
   END IF;

   /******** Calcula os valores: Total Dbitos, Total Crditos, Saldo Credor e Saldo Devedor  ********/
   V_VRTOTADEBI  := NVL(V_VRIMPOSAID,0) +  NVL(V_VROUTRDEBI,0) + NVL(V_VRESTOCRED,0);
   V_VRTOTACRED  := NVL(V_VRIMPOENTR,0) +  NVL(V_VROUTRCRED,0) + NVL(V_VRESTODEBI,0);
   V_VRSALDODEBI := NVL(V_VRTOTADEBI,0) - (NVL(V_VRTOTACRED,0) + NVL(V_VRSALDOCREDANT,0) + NVL(V_VRDEDUCAO,0));

   IF V_VRSALDODEBI < 0 THEN
      V_VRSALDOCRED := ABS(V_VRSALDODEBI);
      V_VRSALDODEBI := 0;
   ELSE
      V_VRSALDOCRED := 0;
   END IF;

   /***** Atualiza a tabela MAPARECUP *****/
   UPDATE MAPARECUP
      SET VRNORMDEBI         = V_VRIMPOSAID,
          VROUTRDEBI         = V_VROUTRDEBI,
          VRESTOCRED         = V_VRESTOCRED,
          VRTOTADEBI         = V_VRTOTADEBI,
          VRNORMCRED         = V_VRIMPOENTR,
          VROUTRCRED         = V_VROUTRCRED,
          VRESTODEBI         = V_VRESTODEBI,
          VRTOTACRED         = V_VRTOTACRED,
          VRSLDCRANTE        = V_VRSALDOCREDANT,
          VRDEDUCAO          = V_VRDEDUCAO,
          VRDIFALIQU         = V_VRDIFALIQU,
          VRSALDOCRED        = V_VRSALDOCRED,
          VRSALDODEBI        = V_VRSALDODEBI,
          VRSLDCRANTESUBAPUR = V_VRSALDOCREDANTSUBA
    WHERE NRSEQUMAPAPU = P_NRSEQUMAPAPU;

   /***** Atualiza a tabela MAPAAPURAC *****/
   UPDATE MAPAAPURAC
      SET VRTTCTBLDEB  = V_VRCONTSAID,
          VRTTBACALDEB = V_VRBASESAID,
          VRTTCTBLCRE  = V_VRCONTENTR,
          VRTTBACALCRE = V_VRBASEENTR,
          VRTOTAIMPOS  = V_VRSALDODEBI,
          IDFORMAPUR   = V_IDFORMAPURNFE
    WHERE NRSEQUMAPAPU = P_NRSEQUMAPAPU;

    /**** VERIFICA SE AS FILIAIS DO MAPA, DEPOIS VERIFICA SE PARA ESSAS FILIAIS EXISTEM NOTAS DE OUTROS MAPAS OU NAO, QUE POSSUEM INSCRICAO DIFERENTE, CASO TENHA, A APURAO DO IMPOSTO NO E PERMITIDA PELO SISTEMA MOBILE ***/
   IF P_CHAMADAPROC = 'M' AND P_CDIMPOSTO = '00' THEN
      V_EMPRRELAC := 'N';
      FOR C_FILIAL IN (SELECT DISTINCT F.CDFILIAL, F.CDEMPRESA, F.CDINSCESTA
                         FROM NOTAFISCAL N, IMPOSTONF I, (SELECT EC.CDFILIAL, EC.CDEMPRESA, EC.CDINSCESTA
                                                            FROM EMPRCNTRFIL EC, FILIAL FI
                                                           WHERE FI.CDEMPRESA  = P_CDEMPRESA
                                                             AND FI.CDINSCESTA = P_INSCRICAO
                                                             AND FI.CDFILIAL   = EC.CDFILIAL
                                                             AND EC.CDEMPRESA||EC.CDINSCESTA <> P_CDEMPRESA||P_INSCRICAO
                                                           UNION ALL
                                                          SELECT FI.CDFILIAL, FI.CDEMPRESA, FI.CDINSCESTA
                                                            FROM EMPRCNTRFIL EC, FILIAL FI
                                                           WHERE EC.CDEMPRESA  = P_CDEMPRESA
                                                             AND EC.CDINSCESTA = P_INSCRICAO
                                                             AND EC.CDFILIAL   = FI.CDFILIAL
                                                             AND FI.CDEMPRESA||FI.CDINSCESTA <> P_CDEMPRESA||P_INSCRICAO) F
                        WHERE N.CDFILIAL     = I.CDFILIAL
                          AND N.NRLANCTONF   = I.NRLANCTONF
                          AND I.NRSEQUMAPAPU = P_NRSEQUMAPAPU
                          AND DECODE(N.IDENTRSAIDOP,'E',N.CDFILIRECE,N.CDFILIEMIT) = F.CDFILIAL) LOOP

         V_EMPRRELAC := 'N';
         FOR C_NFRELAC IN (SELECT 'S' AS IDEMPRRELAC
                             FROM NOTAFISCAL N, IMPOSTONF IP, IMPOSTO IM
                            WHERE IDSTATUS        IN ('A','E')
                              AND IP.CDFILIAL     = N.CDFILIAL
                              AND IP.NRLANCTONF   = N.NRLANCTONF
                              AND N.CDEMPRESA     = C_FILIAL.CDEMPRESA
                              AND N.CDINSCESTA    = C_FILIAL.CDINSCESTA
                              AND N.IDENTRSAIDOP  = 'E'
                              AND N.CDFILIRECE    = C_FILIAL.CDFILIAL
                              AND NVL(IP.DTREFMAPAPU,N.DTENTRSAID) BETWEEN V_DTININFFORPER AND P_DTFINAL
                              AND IP.CDIMPOSTO    = IM.CDIMPOSTO
                              AND IM.CDIMPOSTONFE = P_CDIMPOSTO
                            UNION ALL
                           SELECT 'S' AS IDEMPRRELAC
                             FROM NOTAFISCAL N, IMPOSTONF IP, IMPOSTO IM
                            WHERE IDSTATUS        IN ('A','E')
                              AND IP.CDFILIAL     = N.CDFILIAL
                              AND IP.NRLANCTONF   = N.NRLANCTONF
                              AND N.CDEMPRESA     = C_FILIAL.CDEMPRESA
                              AND N.CDINSCESTA    = C_FILIAL.CDINSCESTA
                              AND N.IDENTRSAIDOP  = 'S'
                              AND N.CDFILIEMIT    = C_FILIAL.CDFILIAL
                              AND NVL(IP.DTREFMAPAPU,N.DTEMISSAO) BETWEEN V_DTININFFORPER AND P_DTFINAL
                              AND IP.CDIMPOSTO    = IM.CDIMPOSTO
                              AND IM.CDIMPOSTONFE = P_CDIMPOSTO) LOOP
             V_EMPRRELAC := 'S';
             EXIT;
         END LOOP;
         IF V_EMPRRELAC = 'S' THEN
            EXIT;
         END IF;
      END LOOP;
      RETURN(V_NFFORAPERIODO||V_EMPRRELAC);
   ELSE
      RETURN(V_NFFORAPERIODO);
   END IF;
END;
/