function MessageManager(MessageService, ScreenService, ZhUtil, ZHPromise) {

    /* global Util */
    /* global Constants */
    /* global Configuration */
    
    const self = this;

    /*******************
     * Message Manager *
     *******************/
    
    const messageManager = {
        processes: [],
        parameters: {},
        status: "success",
        usingCodes: true,
        subscribe: function(processName) {
            this.processes.push({
                name: processName,
                hasNotified: false,
                messages: [],
            });
        },
        unsubscribe: function(processName) {
            this.processes = this.processes.filter(process => process.name != processName);
        },
        unsubscribeAll: function() {
            this.processes = [];
        },
        removeParameter: function(param) {
            if (param in this.parameters) {
                delete this.parameters[param];
            }
        },
        removeAllParameters: function() {
            this.parameters = {};
        },
        resetStatus: function() {
            this.status = "success";
        },
        setStatus: function(status) {
            if (this.status == "success") {
                this.status = status;
            }
        },
        setUsingCodes: function(value) {
            this.usingCodes = value;    
        },
        getMessagesByCodes: function(codes) {
            let promises = [];
            
            for (let code of codes) {
                promises.push(MessageService.getMessageByCode(code[0]));
            }
    
            return ZHPromise.all(promises);
        },
        showMessages: function() {
            const messages = this.processes.filter(process => !Util.isEmptyOrBlank(process.messages)).map(process => process.messages);
            const separator = messages.length > 1 ? '<br><br>' : '';
            
            if (this.usingCodes) {
                const constants = self.getConstants(this.status);
                
                this.getMessagesByCodes(messages).then(function(callback) {
                    const message = callback.join(separator);
                    ScreenService.showMessage(message, constants.type, constants.icon, this.parameters);
                    self.resetDefaults();
                }.bind(this));
            } else {
                const message = messages.join(separator);
                ScreenService.showMessage(message);
                self.resetDefaults();
            }
        }
    };
    
    /******************
     * Funções Gerais *
     ******************/
    
    this.registerProcesses = function(processes) {
        processes.forEach(processName => {
            messageManager.subscribe(processName);
        });
    };
    
    this.notify = function(processName, message, status) {
        const process = messageManager.processes.filter(process => process.name == processName).shift();
        status = status || "success";
        
        process.hasNotified = true;
        
        messageManager.setStatus(status);
        
        if (message) {
            process.messages.push(message);
        }
        
        if (messageManager.processes.every(process => process.hasNotified)) {
            messageManager.showMessages();
        }
    };
    
    this.resetDefaults = function() {
        messageManager.unsubscribeAll();
        messageManager.removeAllParameters();
        messageManager.resetStatus();
    };
    
    this.setParameter = function(key, value) {
        messageManager.parameters[key] = value;
    };
    
    this.setParameters = function(params) {
        messageManager.parameters = params;
    };
    
    this.getConstants = function(status) {
        switch (status) {
            case 'success':
                return Constants.successNotification;
            
            case 'error':
                return Constants.errorMessage;
            
            case 'alert':
                return Constants.alertMessage;
        }
    };
    
    this.showErrorMessage = function(response) {
        let errorMessage = 'Timeout';
                
        if (response.data) {
            errorMessage = response.data.error;
        }
        
        ScreenService.showMessage(errorMessage, Constants.errorMessage.type, Constants.errorMessage.icon, null);
        
        if (!Util.isEmptyOrBlank(messageManager.processes)) {
            self.resetDefaults();
        }
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('MessageManager', MessageManager);
});
